{
    

  HeaderBox  ( )

  Roman M. Mochalov
  E-mail: roman@infra.sar.nnov.ru
}

unit Ex_Header;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

{ Predefine classes }

  THeaderBoxSections = class;
  THeaderBox = class;

{ THeaderBoxSection }

  THeaderBoxSection = class(TCollectionItem)
  private
    FParent: THeaderBoxSections;
    FSections: THeaderBoxSections;
    FCaption: string;
    FWidth: Integer;
    FAlignment: TAlignment;
    function GetLevel: Integer;
    procedure SetAlignment(Value: TAlignment);
    procedure SetCaption(const Value: string);
    procedure SetSections(Value: THeaderBoxSections);
    procedure SetWidth(Value: Integer);
  protected
    procedure Change; virtual;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Level: Integer read GetLevel;
    property Parent: THeaderBoxSections read FParent;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Caption: string read FCaption write SetCaption;
    property Width: Integer read FWidth write SetWidth;
    property Sections: THeaderBoxSections read FSections write SetSections;
  end;

{ THeaderBoxSections }

  THeaderBoxSections = class(TCollection)
  private
    FOwner: THeaderBox;
    FSection: THeaderBoxSection;
    function GetSection(Index: Integer): THeaderBoxSection;
    procedure SetSection(Index: Integer; Value: THeaderBoxSection);
  protected
    procedure Update(Section: TCollectionItem); override;
  public
    constructor Create(AOwner: THeaderBox; ASection: THeaderBoxSection);
    function Add: THeaderBoxSection;
    property Owner: THeaderBox read FOwner;
    property Section: THeaderBoxSection read FSection;
    property Sections[Index: Integer]: THeaderBoxSection read GetSection write SetSection; default;
  end;

{ THeaderBox }

  THeaderBox = class(TCustomControl)
  private
    FSections: THeaderBoxSections;
    FSectionHeight: Integer;
    FAutoSize: Boolean;
    procedure SetAutoSize(Value: Boolean);
    procedure SetSections(Value: THeaderBoxSections);
    procedure SetSectionHeight(Value: Integer);
  protected
    procedure CalckSize; virtual;
    procedure Loaded; override;
    procedure Paint; override;
    procedure PaintSection(Section: THeaderBoxSection; Rect: TRect); virtual;
    procedure PaintSections(Sections: THeaderBoxSections; Rect: TRect); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property Color;
    property Font;
    property Sections: THeaderBoxSections read FSections write SetSections;
    property SectionHeight: Integer read FSectionHeight write SetSectionHeight default 15;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{ THeaderBoxSection }

constructor THeaderBoxSection.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FParent := THeaderBoxSections(Collection);
  FSections := THeaderBoxSections.Create(FParent.Owner, Self);
  FWidth := 64;
  FAlignment := taLeftJustify;
end;

destructor THeaderBoxSection.Destroy;
begin
  FSections.Free;
  inherited Destroy;
end;

function THeaderBoxSection.GetLevel: Integer;

  function CalckLevel(Sections: THeaderBoxSections; Level: Integer): Integer;
  begin
    if Sections = nil then
      Result := Level else
      Result := CalckLevel(Sections.Section.Parent, Level + 1);
  end;

begin
  Result := CalckLevel(Parent, 0);
end;

procedure THeaderBoxSection.SetAlignment(Value: TAlignment);
begin
  if Alignment <> Value then
  begin
    FAlignment := Value;
    Change;
  end;
end;

procedure THeaderBoxSection.SetCaption(const Value: string);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Change;
  end;
end;

procedure THeaderBoxSection.SetSections(Value: THeaderBoxSections);
begin
  FSections.Assign(Value);
end;

procedure THeaderBoxSection.SetWidth(Value: Integer);
begin
  if (Value > 0) and (Width <> Value) then
  begin
    FWidth := Value;
    Change;
  end;
end;

procedure THeaderBoxSection.Change;
begin
  with Parent do
    if (Owner <> nil) and Owner.AutoSize then Owner.CalckSize;
  Changed(False);
end;

procedure THeaderBoxSection.Assign(Source: TPersistent);
var
  Section: THeaderBoxSection absolute Source;
begin
  if Source is THeaderBoxSection then
  begin
    Sections := Section.Sections;
    Caption := Section.Caption;
    Width := Section.Width;
    Alignment := Section.Alignment;
    Exit;
  end;
  inherited Assign(Source);
end;

{ THeaderBoxSections }

constructor THeaderBoxSections.Create(AOwner: THeaderBox; ASection: THeaderBoxSection);
begin
  inherited Create(THeaderBoxSection);
  FOwner := AOwner;
  FSection := ASection;
end;

function THeaderBoxSections.GetSection(Index: Integer): THeaderBoxSection;
begin
  Result := THeaderBoxSection(inherited GetItem(Index));
end;

procedure THeaderBoxSections.SetSection(Index: Integer; Value: THeaderBoxSection);
begin
  inherited SetItem(Index, Value);
end;

procedure THeaderBoxSections.Update(Section: TCollectionItem);
begin
  if (Section <> nil) and (Owner <> nil) then Owner.Repaint;
end;

function THeaderBoxSections.Add: THeaderBoxSection;
begin
  Result := THeaderBoxSection(inherited Add);
end;

{ THeaderBox }

constructor THeaderBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alTop;
  Height := 16;
  Color := clBtnFace;
  FSections := THeaderBoxSections.Create(Self, nil);
  FSectionHeight := 15;
  FAutoSize := True;
end;

destructor THeaderBox.Destroy;
begin
  FSections.Free;
  inherited Destroy;
end;

procedure THeaderBox.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize := Value;
    CalckSize;
  end;
end;

procedure THeaderBox.SetSections(Value: THeaderBoxSections);
begin
  FSections.Assign(Value);
end;

procedure THeaderBox.SetSectionHeight(Value: Integer);
begin
  if FSectionHeight <> Value then
  begin
    FSectionHeight := Value;
    if AutoSize then
      CalckSize else
      Repaint;
  end;
end;

procedure THeaderBox.CalckSize;
var
  W, L: Integer;

  function CalckWidth(Sections: THeaderBoxSections): Integer;
  var
    I: Integer;
  begin
    Result := 0;
    for I := 0 to Sections.Count - 1 do
      Result := Result + Sections[I].Width + 2;
  end;

  function CalckLevel(Sections: THeaderBoxSections; Level: Integer): Integer;
  var
    I, L: Integer;
  begin
    Result := Level;
    for I := 0 to Sections.Count - 1 do
    begin
      L := CalckLevel(Sections[I].Sections, Level + 1);
      if Result < L then Result := L;
    end;
  end;

begin
  if Sections.Count > 0 then
  begin
    W := CalckWidth(Sections);
    L := CalckLevel(Sections, 0);
  end else
  begin
    W := Width;
    L := 1;
  end;
  SetBounds(Left, Top, W, L * (SectionHeight + 2) - 1);
end;

procedure THeaderBox.Loaded;
begin
  if AutoSize then CalckSize;
  inherited Loaded;
end;

procedure THeaderBox.Paint;
var
  R: TRect;
begin                                 
  Canvas.Brush.Color := Color;
  Canvas.Font := Font;
  {  ,    }
  if Sections.Count = 0 then Exit;
  {   }
  with Canvas do
  begin
    Pen.Color := clBtnShadow;
    MoveTo(0, Height - 1);
    LineTo(Width, Height - 1);
  end;
  {  }
  R := Bounds(0, 0, Width, Height);
  PaintSections(Sections, R);
end;

procedure THeaderBox.PaintSection(Section: THeaderBoxSection; Rect: TRect);
const
  D = DT_SINGLELINE or DT_VCENTER or DT_END_ELLIPSIS;
var
  R: TRect;
  F: Integer;
  P: TDrawTextParams;
  T: string;
begin
  {   }
  with Canvas do
  begin
    {   }
    Pen.Color := clBtnShadow;
    MoveTo(Rect.Right - 2, Rect.Top - 1);
    LineTo(Rect.Right - 2, Rect.Bottom);
    Pen.Color := clBtnHighlight;
    MoveTo(Rect.Right - 1, Rect.Bottom);
    LineTo(Rect.Right - 1, Rect.Top - 1);
    {  }
    R.Left := Rect.Left;
    R.Right := Rect.Right - 2;
    R.Top := Rect.Top + 2;
    if Section.Sections.Count > 0 then
      R.Bottom := Rect.Top + SectionHeight else
      R.Bottom := Rect.Bottom;
    P.cbSize := SizeOf(P);
    P.iTabLength := 0;
    P.iLeftMargin := 6;
    P.iRightMargin := 6;
    P.uiLengthDrawn := 0;
    F := D;
    case Section.Alignment of
      taLeftJustify: F := D or DT_LEFT;
      taCenter: F := D or DT_CENTER;
      taRightJustify: F := D or DT_RIGHT;
    end;
    T := Section.Caption;
    Brush.Style := bsClear;
    DrawTextEx(Handle, PChar(T), Length(T), R, F, @P);
  end;
  {  }
  if Section.Sections.Count > 0 then
  begin
    {    }
    Rect.Top := Rect.Top + SectionHeight + 2;
    {   }
    with Canvas do
    begin
      Pen.Color := clBtnShadow;
      MoveTo(Rect.Left, Rect.Top - 2);
      LineTo(Rect.Right - 1, Rect.Top - 2);
      Pen.Color := clBtnHighlight;
      MoveTo(Rect.Left, Rect.Top - 1);
      LineTo(Rect.Right - 1, Rect.Top - 1);
    end;
    PaintSections(Section.Sections, Rect);
  end;
end;

procedure THeaderBox.PaintSections(Sections: THeaderBoxSections; Rect: TRect);
var
  I: Integer;
  R: TRect;
  B: Boolean;
begin
  {  ,    }
  if Sections.Count = 0 then Exit;
  B := False;
  {  }
  R.Left := Rect.Left;
  R.Right := Rect.Left;
  R.Top := Rect.Top;
  R.Bottom := Rect.Bottom;
  for I := 0 to Sections.Count - 1 do
  begin
    R.Left := R.Right + 1;
    R.Right := R.Left + Sections[I].Width;
    if R.Right > Rect.Right then
    begin
      R.Right := Rect.Right;
      B := True;
    end;
    PaintSection(Sections[I], R);
    if B then Break;
  end;
end;

end.
