{
    

     THeader

   . 
  E-mail: roman@sar.nnov.ru
}

unit Ex_HeaderE;                

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Graphics, StdCtrls, ComCtrls,
  Ex_Header;

type

{ TSectionsEditorForm }

  TSectionsEditorForm = class(TForm)
    SectionsGroup: TGroupBox;
    SectionsTree: TTreeView;
    AddButton: TButton;
    DeleteButton: TButton;
    PropertiesGroup: TGroupBox;
    IndexLabel: TLabel;
    IndexEdit: TEdit;
    CaptionLabel: TLabel;
    CaptionEdit: TEdit;
    WidthLabel: TLabel;
    WidthEdit: TEdit;
    AlignmentLabel: TLabel;
    AlignmentCombo: TComboBox;
    OKButton: TButton;
    CancelButton: TButton;
    ApplyButton: TButton;
    procedure EnableApply(Sender: TObject);
    procedure DisableApply(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SectionsTreeChange(Sender: TObject; Node: TTreeNode);
    procedure SectionsTreeChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
    procedure SectionsTreeEnter(Sender: TObject);
    procedure SectionsTreeKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AddButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
  private
    FHeader: THeaderBox;
    FSections: THeaderBoxSections;
    FExceptCount: Integer;
    procedure AddSection;
    procedure DeleteSection;
    procedure CheckSection;
    procedure GetSection;
    procedure GetParams;
    procedure PutSection;
    procedure PutParams;
    procedure RefreshView;
  public
    function Execute(Header: THeaderBox): Boolean;
  end;

function EditHeaderBoxSections(Header: THeaderBox): Boolean;

implementation

{$R *.DFM}

function EditHeaderBoxSections(Header: THeaderBox): Boolean;
begin
  with TSectionsEditorForm.Create(nil) do
  try
    Result := Execute(Header);
  finally
    Free;
  end;
end;

{ Exception handlers }

type
  EHeaderError = class(Exception);

procedure IndexError(Index: string);
begin
  raise EHeaderError.CreateFmt('"%s"   ', [Index]);
end;

procedure WidthError(Width: string);
begin
  raise EHeaderError.CreateFmt('"%s"   ', [Width]);
end;

{ THeadersEditorForm }

procedure TSectionsEditorForm.AddSection;
var
  C: THeaderBoxSection;
begin
  with SectionsTree do
    if Selected <> nil then
      C := THeaderBoxSection(Selected.Data) else
      C := nil;
  if C = nil then
    FSections.Add else
    C.Sections.Add;
end;

procedure TSectionsEditorForm.DeleteSection;
var
  C: THeaderBoxSection;
begin
  with SectionsTree do
    if Selected <> nil then
      C := THeaderBoxSection(Selected.Data) else
      C := nil;
  if C <> nil then C.Free;
end;

procedure TSectionsEditorForm.CheckSection;
begin
  if PropertiesGroup.Enabled then
  begin
    with IndexEdit do
    try
      if StrToIntDef(Text, -1) < 0 then IndexError(Text);
    except
      SetFocus;
      raise;
    end;
    with WidthEdit do
    try
      if StrToIntDef(Text, -1) < 0 then WidthError(Text);
    except
      SetFocus;
      raise;
    end;
  end;
end;

procedure TSectionsEditorForm.GetSection;
var
  C: THeaderBoxSection;
begin
  with SectionsTree do
    if Selected <> nil then
      C := THeaderBoxSection(Selected.Data) else
      C := nil;
  if C <> nil then
  begin
    IndexEdit.Text := IntToStr(C.Index);
    CaptionEdit.Text := C.Caption;
    WidthEdit.Text := IntToStr(C.Width);
    AlignmentCombo.ItemIndex := Ord(C.Alignment);
  end;
end;

procedure TSectionsEditorForm.GetParams;
begin
  FSections.Assign(FHeader.Sections);
  RefreshView;
end;

procedure TSectionsEditorForm.PutSection;
var
  C: THeaderBoxSection;
begin
  with SectionsTree do
    if Selected <> nil then
      C := THeaderBoxSection(Selected.Data) else
      C := nil;
  if C <> nil then
  begin
    C.Index := StrToInt(IndexEdit.Text);
    C.Caption := CaptionEdit.Text;
    C.Width := StrToInt(WidthEdit.Text);
    C.Alignment := TAlignment(AlignmentCombo.ItemIndex);
  end;
end;

procedure TSectionsEditorForm.PutParams;
begin
  FHeader.Sections := FSections;
end;

procedure TSectionsEditorForm.RefreshView;

  procedure BeginRefresh;
  begin
    SectionsTree.OnChange := nil;
    SectionsTree.OnChanging := nil;
    IndexEdit.OnChange := nil;
    CaptionEdit.OnChange := nil;
    WidthEdit.OnChange := nil;
    AlignmentCombo.OnChange := nil;
  end;

  procedure EndRefresh;
  begin
    SectionsTree.OnChange := SectionsTreeChange;
    SectionsTree.OnChanging := SectionsTreeChanging;
    IndexEdit.OnChange := EnableApply;
    CaptionEdit.OnChange := EnableApply;
    WidthEdit.OnChange := EnableApply;
    AlignmentCombo.OnChange := EnableApply;
  end;

  procedure RefreshTree;

    procedure ProcessNode(Node: TTreeNode; Sections: THeaderBoxSections);
    var
      I: Integer;
      R: TRect;
    begin
      if Node.Count > Sections.Count then
      begin
        I := Node.Count;
        while I > Sections.Count do
        begin
          Dec(I);
          Node.Item[I].Delete;
        end;
      end;
      if Node.Count < Sections.Count then
      begin
        I := Node.Count;
        while I < Sections.Count do
        begin
          Inc(I);
          with SectionsTree do Selected := Items.AddChild(Node, '');
        end;
      end;
      for I := 0 to Node.Count - 1 do
      begin
        if CompareText(Node.Item[I].Text, Sections[I].Caption) <> 0 then
        begin
          Node.Item[I].Text := Sections[I].Caption;
          R := Node.Item[I].DisplayRect(True);
          InvalidateRect(SectionsTree.Handle, @R, False);
        end;
        Node.Item[I].Data := Sections[I];
      end;
      for I := 0 to Node.Count - 1 do
      begin
        ProcessNode(Node.Item[I], Sections[I].Sections);
      end;
    end;

  begin
    ProcessNode(SectionsTree.Items.GetFirstNode, FSections);
  end;

  procedure RefreshControls;
  var
    I: THeaderBoxSection;
  begin
    with SectionsTree do
      if Selected <> nil then
        I := THeaderBoxSection(Selected.Data) else
        I := nil;
    if I <> nil then
    begin
      PropertiesGroup.Enabled := True;
      DeleteButton.Enabled := True;
      GetSection;
    end else
    begin
      WidthEdit.Text := '';
      CaptionEdit.Text := '';
      IndexEdit.Text := '';
      AlignmentCombo.ItemIndex := -1;
      DeleteButton.Enabled := False;
      PropertiesGroup.Enabled := False;
    end;
  end;

begin
  BeginRefresh;
  try
    RefreshTree;
    RefreshControls;
  finally
    Endrefresh;
  end;
end;

function TSectionsEditorForm.Execute(Header: THeaderBox): Boolean;
begin
  FHeader := Header;
  GetParams;
  RefreshView;
  Result := ShowModal = mrOK;
end;

procedure TSectionsEditorForm.EnableApply(Sender: TObject);
begin
  OKButton.Default := False;
  ApplyButton.Enabled := True;
  ApplyButton.Default := True;
end;

procedure TSectionsEditorForm.DisableApply(Sender: TObject);
begin
  ApplyButton.Default := False;
  ApplyButton.Enabled := False;
  OKButton.Default := True;
end;

procedure TSectionsEditorForm.FormCreate(Sender: TObject);
begin
  FSections := THeaderBoxSections.Create(nil, nil);
  SectionsTree.Items.Add(nil, '');
end;

procedure TSectionsEditorForm.FormDestroy(Sender: TObject);
begin
  FSections.Free;
end;

procedure TSectionsEditorForm.SectionsTreeChange(Sender: TObject; Node: TTreeNode);
begin
  if not (csDestroying in ComponentState) then RefreshView;
end;

procedure TSectionsEditorForm.SectionsTreeChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);
begin
  if not (csDestroying in ComponentState) then
  begin
    try
      CheckSection;
      PutSection;
      RefreshView;
    except
      if FExceptCount = 0 then Application.HandleException(Self);
      Inc(FExceptCount);
    end;
    AllowChange := FExceptCount = 0;
  end;
end;

procedure TSectionsEditorForm.SectionsTreeEnter(Sender: TObject);
begin
  FExceptCount := 0;
end;

procedure TSectionsEditorForm.SectionsTreeKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_INSERT: with AddButton do if Enabled then Click;
    VK_DELETE: with DeleteButton do if Enabled then Click
  end;
end;

procedure TSectionsEditorForm.AddButtonClick(Sender: TObject);
begin
  CheckSection;
  PutSection;
  AddSection;
  RefreshView;
  CaptionEdit.SetFocus;
  EnableApply(Sender);
end;

procedure TSectionsEditorForm.DeleteButtonClick(Sender: TObject);
begin
  CheckSection;
  PutSection;
  DeleteSection;
  RefreshView;
  SectionsTree.SetFocus;
  EnableApply(Sender);
end;

procedure TSectionsEditorForm.OKButtonClick(Sender: TObject);
begin
  CheckSection;
  PutSection;
  PutParams;
  ModalResult := mrOK;
end;

procedure TSectionsEditorForm.ApplyButtonClick(Sender: TObject);
begin
  CheckSection;
  PutSection;
  PutParams;
  GetParams;
  DisableApply(nil)
end;

end.
